#ifndef SATLIB_H_
#define SATLIB_H_

#define	SAT_MAX_VALUE	INT32_MAX
#define	SAT_MIN_VALUE	INT32_MIN

#define	SAT_OVERFLOW	0x1
#define	SAT_BY_ZERO	0x2

#define SAT_PI		0x0003243f
#define	SAT_PI2		0x0001921f
#define	SAT_ONE		0x00010000
#define	SAT_ZERO	0x00000000

#define	SAT_TOO_LONG	1;

#include <stdint.h>

/* if an exception occurs, appropriate flag is set here;
 * all flags are omitted by default
 */
uint32_t sat_flags;

/* simply saturated addition;
 * sets SAT_OVERFLOW flag on overflow
 */
int32_t sat_add(int32_t a, int32_t b);

/* simply saturated subtraction;
 * sets SAT_OVERFLOW flag on overflow
 */
int32_t sat_sub(int32_t a, int32_t b);

/* saturated multiplication;
 * it uses Karatsuba algorithm for simplicity
 * sets SAT_OVERFLOW flag on overflow
 */
int32_t sat_mul(int32_t a, int32_t b);

/* saturated division;
 * sets SAT_OVERFLOW flag on overflow;
 */
int32_t sat_div(int32_t a, int32_t b);

/* returns sine of number */
int32_t sat_sin(int32_t x);

/* returns cosine of number */
int32_t sat_cos(int32_t x);

/* implementation of Fast Fourier Transform;
 * inv:	0 - direct,
 *	1 - inverse;
 * !!! ATTENTOIN !!!
 * number of complex points 'n' must be less than 16384;
 * returns SAT_TOO_LONG if 'n' is greater 16384;
 * returns 0 if everything is good;
 * sets SAT_OVERFLOW and SAT_BY_ZERO flags on appropriate exceptions;
 */
int sat_fft(int32_t *x, int n, int inv);

#endif

